package org.coows;

import java.util.Vector;

/**
 * Class for ONLY_TRUSTED access specifier type
 */
public class CooAccessSpecifierOnlyTrusted 
	extends CooAbstractAccessSpecifier {
	
    /** Agents with grant access to the plan */
	public Vector<String> mTrustedAgents;

	/** Constructor */
	public CooAccessSpecifierOnlyTrusted() {
		super("ONLY_TRUSTED");
		this.mTrustedAgents = new Vector<String>();
	}


	/**
	 * Constructor
	 * 
	 * @param aAccSpec a list of agents ID to grant
	 */
	public CooAccessSpecifierOnlyTrusted(Vector<String> aTrustedAgentsList) {
		super("ONLY_TRUSTED");
		this.mTrustedAgents = aTrustedAgentsList;
	}


	/**
	 * Update the set of trusted agents with a new agent 
	 * 
	 * @param anAgentID the agent to trust
	 */
	public void trustAgent(String anAgentID) {
		if (!mTrustedAgents.contains(anAgentID))
			mTrustedAgents.add(anAgentID);
	}


	/**
	 * @see ICooAccessSpecifier#isTrusted(java.lang.String)
	 */
	public boolean isTrusted(String anAgentID) {
		if (mTrustedAgents.contains(anAgentID))
			return true;
		else
			return false;
	}
}
