package org.coows;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.LinkedList;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.coows.engine.CooEngine;
import org.coows.engine.CooException;
import org.coows.work.CooWorkerProcessEvent;
import org.coows.work.CooWorkerProcessEventFactory;

public class CooAgentEventsQueue {
    /**
     * Logger for this class
     */
    private static final Log logger =
        LogFactory.getLog(CooAgentEventsQueue.class);
    
    private LinkedList<ICooEvent> mQueuedEvents;
    private int mMaxActiveWorkers;
    private GenericObjectPool mWorkersPool;

    /**
     * 
     * @uml.property name="mAgentOwner"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private CooAgent mAgentOwner;

    
    /**
     * The pool of thread worker
     */
    private GenericObjectPool mDispatchersPool;
    
    public CooAgentEventsQueue(CooAgent anAgent)
        throws CooException {
        try {
            mWorkersPool = new GenericObjectPool(
                    new CooWorkerProcessEventFactory());
            mMaxActiveWorkers = Integer.parseInt(CooEngine.getMConfig()
                    .getText("/config/pools/agentWorkers/maxActive"));
            mWorkersPool.setMaxActive(mMaxActiveWorkers);
            mAgentOwner = anAgent;
            mQueuedEvents = new LinkedList<ICooEvent>();
        } catch(Exception e) {
            throw new CooException("Error initializing EventQueue!");
        }
    }
    
    public void enqueue(ICooEvent anEvent) {
        try {
            if (mMaxActiveWorkers > mWorkersPool.getNumActive()) {
                // process immediatly the new event
                CooWorkerProcessEvent w = (CooWorkerProcessEvent)
                    mWorkersPool.borrowObject();
                w.setPool(mWorkersPool);
                w.processEvent(mAgentOwner, anEvent);
            }
            else {
                // enqueue the event
                synchronized (mQueuedEvents) {
                    mQueuedEvents.add(anEvent);
                }

                if (logger.isDebugEnabled()) {
                    logger.debug("enqueue() - Event " + anEvent +"enqueued!");
                }
            }
        } catch (Exception e) {
            if (logger.isErrorEnabled())
                logger.error("agent: " + mAgentOwner 
                    +"onEvent(ICooEvent) error processing " + anEvent, e);
        }

    }

    /**
     * 
     * @uml.property name="mQueuedEvents"
     */
    public LinkedList<ICooEvent> getMQueuedEvents() {
        return mQueuedEvents;
    }

}
