package org.coows;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.coows.engine.CooEngine;
import org.coows.engine.CooException;

import org.coows.util.CooBprFile;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.rpc.ParameterMode;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.encoding.XMLType;

/**
 * Class container for a binary .bpr file
 */
public class CooBpelProcessBpr
	implements ICooBpelProcess {

    /**
     * The binary content of the .bpr file
     * 
     * @uml.property name="mBprFile"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private CooBprFile mBprFile;

	/** The service name of the process contained into the .bpr file */
	private String mServiceName;


	/**
	 * Constructor
	 * 
	 */
	public CooBpelProcessBpr() {
	}


	/**
	 * Constructor
	 * 
	 * @param aBprFile
	 * @param aServiceName
	 */
	public CooBpelProcessBpr(CooBprFile aBprFile, String aServiceName) {
		this.mBprFile = aBprFile;
		this.mServiceName = aServiceName;
	}


	public void setBprFile(CooBprFile aBprFile) {
		this.mBprFile = aBprFile;
	}


	public CooBprFile getBprFile() {
		return mBprFile;
	}


	public void setServiceName(String aServiceName) {
		this.mServiceName = aServiceName;
	}


	public String getServiceName() {
		return mServiceName;
	}


	/**
	 * Deploys the process to the ActiveBPEL engine
	 * 
	 * @throws CooException
	 */
	public void deploy() throws CooException {
		String bprDir = CooEngine.CATALINA_HOME + "/bpr";

		try {
			// Deploy == Crete che .bpr file into the bpr directory
			FileOutputStream fos = new FileOutputStream(bprDir + "/"
					+ mServiceName + ".bpr");
			byte[] contentFile = new byte[mBprFile.getBin().available()];
			mBprFile.getBin().read(contentFile);
			fos.write(contentFile);
			fos.close();
		} catch (FileNotFoundException e) {
			throw new CooException(" Unable to deploy process " + mServiceName
					+ "\n" + e.getMessage());
		} catch (IOException e) {
			throw new CooException(" Unable to deploy process " + mServiceName
					+ "\n" + e.getMessage());
		}
	}


	/**
	 * @throws CooException
	 * @see org.coows.ICooBpelProcess#invoke(org.coows.CooBpelProcessParameters)
	 */
	public void invoke(CooBpelProcessParameters p) throws CooException {
		Call call;
		try {
			call = createInvokeCall(p);
			call.invoke(new Object[] { p.getAgentInvoker() });
		} catch (Exception e) {
			throw new CooException("[CooBpelProcessBpr]"
					+ " Error invoking bpel process " + mServiceName + "\n"
					+ e.getLocalizedMessage());
		}
	}


	public Call createInvokeCall(CooBpelProcessParameters params)
			throws ServiceException, MalformedURLException {
		Call call = (Call) new Service().createCall();
		call.setTargetEndpointAddress(new URL(
				"http://localhost:8080/active-bpel/services/" + mServiceName));
		call.setOperationName("runPlanOp");
		// Registering type mapping
		// call.registerTypeMapping(CooDesire.class,
		// new QName("urn:coows", "Desire"),
		// BeanSerializerFactory.class, BeanDeserializerFactory.class);

		// Adding parameters to call
		call.addParameter("agentID", XMLType.XSD_STRING, ParameterMode.IN);

		// call.setOperationStyle("rpc");
		// call.setOperationName();

		return call;
	}
    
    /**
     * 
     * @see org.coows.ICooBpelProcess#undeploy()
     */
    public void undeploy() {
       //TODO 
    }
}
