package org.coows;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.LinkedList;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.coows.engine.CooEngine;
import org.coows.engine.CooException;
import org.coows.work.CooWorkerDispatcher;
import org.coows.work.CooWorkerDispatcherFactory;


public class CooGlobalEventsQueue {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory
            .getLog(CooGlobalEventsQueue.class);

    /** 
     * The list of queued event waiting to be dispatched
     */
    private LinkedList<ICooEvent> mQueuedEvents;
    
    /**
     * The number of max active workers allowed 
     */
    private int mMaxActiveWorkers;
    
    /**
     * The pool of threads
     */
    private GenericObjectPool mDispatchersPool;
    
    /**
     * Constructor
     * @throws CooException
     */
    public CooGlobalEventsQueue()
        throws CooException {
        try {
            mDispatchersPool = new GenericObjectPool(
                    new CooWorkerDispatcherFactory());
            mMaxActiveWorkers = Integer.parseInt(CooEngine.getMConfig()
                    .getText("/config/pools/globalDispatchers/maxActive"));
            mDispatchersPool.setMaxActive(mMaxActiveWorkers);
            mQueuedEvents = new LinkedList<ICooEvent>();
        } catch(Exception e) {
            throw new CooException("Error initializing EventQueue!");
        }
    }
    
    /**
     * Enqueue the event
     * @param anEvent
     */
    public void enqueue(ICooEvent anEvent) {
        try {
            if (mMaxActiveWorkers > mDispatchersPool.getNumActive()) {
                CooWorkerDispatcher w = (CooWorkerDispatcher)
                                        mDispatchersPool.borrowObject();
                w.setPool(mDispatchersPool);
                w.dispatchEvent(
                 CooEngine.getMAgentsManager().getAgent(anEvent.getMAgentTO()),
                 anEvent);
            }
            else {
                synchronized (mQueuedEvents) {
                    mQueuedEvents.add(anEvent);
                }
            }
        }
        catch (Exception e) {
            if (logger.isErrorEnabled()) {
                logger.error("enqueue() - Error while dispatching event "
                        + anEvent, e);
            }
        }
    }

    /**
     * 
     * @uml.property name="mQueuedEvents"
     */
    public LinkedList getMQueuedEvents() {
        return mQueuedEvents;
    }

    public void clear() {
        mQueuedEvents.clear();
    }

}
