package org.coows;

import java.util.Calendar;
import java.util.Vector;

import org.coows.engine.CooEngine;
import org.coows.util.CooSetOfPlans;

public class CooInstanceRecord {

    /**
     * Boolean flag for 
     */
    public Boolean mIsSuspended;

    /**
     * 
     * @uml.property name="mCurrentRunningPlan"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    public CooPlan mCurrentRunningPlan;

    /**
     * 
     * @uml.property name="mRelevantPlans"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    public Vector<CooPlan> mRelevantPlans;

    /**
     * 
     * @uml.property name="mFailedPlans"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    public Vector<CooPlan> mFailedPlans;

    /**
     * 
     * @uml.property name="mDes"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private ICooDesire mDes;

    private Calendar mTimeout;

    /**
     * 
     * @uml.property name="mRefToPrev"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private CooInstanceRecord mRefToPrev;

    /**
     * 
     * @uml.property name="mRefToIntention"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private CooIntention mRefToIntention;

    
    
    /**
     * Constructor 
     * @param aDes
     * @param aSop
     */
    public CooInstanceRecord(ICooDesire aDes, CooSetOfPlans aSop) {
        mDes = aDes;
        mRelevantPlans = new Vector<CooPlan>(aSop.getCollection());
        mFailedPlans = new Vector<CooPlan>();
        mTimeout = Calendar.getInstance();
        mTimeout.add(Calendar.SECOND, 20);
        if (mRelevantPlans.isEmpty())
            mIsSuspended = Boolean.TRUE;
        else
            mIsSuspended = Boolean.FALSE;
    }

    /**
     * Receive a set of provided plans
     * @param aSop
     */
    public void provided(CooSetOfPlans aSop) {
        Vector<CooPlan> prov = new Vector<CooPlan>(aSop.getCollection());
        prov.remove(mCurrentRunningPlan);
        prov.removeAll(mFailedPlans);
        
        if (prov.isEmpty())
            return;

        synchronized (mIsSuspended) {
            mRelevantPlans.addAll(prov);
            if (mIsSuspended.booleanValue()) {
                mIsSuspended = Boolean.FALSE;
                runNextPlan();
            }
        }    
    }

    public boolean isInTimeout() {
        return Calendar.getInstance().after(mTimeout);
    }

    public void fail() {
        if (mRefToPrev == null)
            mRefToIntention.fail();
        else
            mRefToPrev.failCurrentPlan();
    }
    
    public void failCurrentPlan() {
        mFailedPlans.add(mCurrentRunningPlan);
        mCurrentRunningPlan = null;
        
        if (!mRelevantPlans.isEmpty()) {
            runNextPlan();
        } else 
            if (isInTimeout())
                this.fail();
            else
                mIsSuspended = Boolean.FALSE;
    }

    /**
     * 
     * @uml.property name="mRefToPrev"
     */
    public void setMRefToPrev(CooInstanceRecord refToPrev) {
        mRefToPrev = refToPrev;
    }

    /**
     * 
     * @uml.property name="mRefToIntention"
     */
    public void setMRefToIntention(CooIntention refToIntention) {
        mRefToIntention = refToIntention;
    }

    
    public void runNextPlan() {
        mCurrentRunningPlan = mRelevantPlans.remove(0);
        
        CooEngine.getMProcessesManager().invokeProcess(
                mRefToIntention.getMAgentOwner(), 
                mRefToIntention.getMIntentionID(),
                mCurrentRunningPlan.getMPlanID());
    }
}
