package org.coows;

import java.util.Vector;
import org.coows.engine.CooEngine;

/**
 * The intention <br>
 * An intention is a stack of "instance records" <br>
 * (@see org.coows.CooInstanceRecord)
 */
public class CooIntention  {
    
    private String mIntentionID;

    /**
     * 
     * @uml.property name="mSuccessActions"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private ICooAction mSuccessActions;

    /**
     * 
     * @uml.property name="mFailureActions"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private ICooAction mFailureActions;

    private Vector<CooInstanceRecord> mStack;
    private Vector<String> mStackRecordIDs;
    private int mCounter = 0;
    private String mAgentOwner;
    private boolean mCompleted;
    

    /**
     * Constructor
     *
     */
    public CooIntention(
            String anAgentID,
            String anIntentionID,
            ICooAction aSuccessAction,
            ICooAction aFailureAction) {
        mAgentOwner = anAgentID;
        mIntentionID = anIntentionID;
        mSuccessActions = aSuccessAction;
        mFailureActions = aFailureAction;
        mStack = new Vector<CooInstanceRecord>();
        mStackRecordIDs = new Vector<String>();
        mCompleted = false;
    }
   
    /**
     * Push a new instance record on top (occurs on "achieve" event)
     * @param anInstanceRecord
     */
    public String push(CooInstanceRecord anInstanceRecord) {
        synchronized (mStack) {
            if (mStack.isEmpty())
                anInstanceRecord.setMRefToPrev(null);
            else
                anInstanceRecord.setMRefToPrev(mStack.lastElement());
            
            anInstanceRecord.setMRefToIntention(this);
            mStack.add(anInstanceRecord);
            mStackRecordIDs.add(String.valueOf(mCounter));
            
            return String.valueOf(mCounter++);
        }
    }
    
    
    public void checkSuspended() {
        int pos = 0;
        boolean isFailed = false;
        
        synchronized (top().mIsSuspended) {
            if (top().mIsSuspended.booleanValue() && top().isInTimeout()) {
                pos = mStack.indexOf(top());
                mStack.remove(pos);
                isFailed = true;
            }
        }
        
        if (isFailed)
            if (pos==0)
                fail();
            else {
                CooInstanceRecord prevInst =
                    (CooInstanceRecord) mStack.elementAt(pos-1);
                prevInst.failCurrentPlan();
            }
    }

    /**
     * 
     * @uml.property name="mAgentOwner"
     */
    public String getMAgentOwner() {
        return mAgentOwner;
    }

    /**
     * 
     * @uml.property name="mIntentionID"
     */
    public String getMIntentionID() {
        return mIntentionID;
    }

    public void fail() {
        mFailureActions.exec();
        mCompleted = true;        
    }

    public void dispatchPlanOutcome(CooEventPlanOutcome anEvent) {
        if (anEvent.isSuccess()) {
            mStack.remove(top());
            if (mStack.size() == 0) {
                mSuccessActions.exec();
                mCompleted = true;
            } else
                CooEngine.getMProcessesManager().dispatchAchieved(mAgentOwner,
                        mIntentionID, top().mCurrentRunningPlan.getMPlanID());
        } else
            top().failCurrentPlan();
    }
    
    public void dispatchProvided(CooEventProvided anEvent) {
        CooInstanceRecord dest;
        
        if (mStackRecordIDs.contains(anEvent.getMReqID().getMInstanceID())) {
                dest = mStack.get(mStackRecordIDs.indexOf(
                        anEvent.getMReqID().getMInstanceID()));
                dest.provided(anEvent.getMProvidedPlans());
        }
    }
    
    public CooInstanceRecord top() {
        return mStack.lastElement();
    }
    
    public boolean isCompleted() {
        return mCompleted;
    }
}
