package org.coows;

/**
 * A CooWS plan is defined by the tuple
 * (plan ID, trigger, body, access specifier)
 * 
 */
public class CooPlan {
	/**
	 * The unique plan identifier
	 */
	private String mPlanID;

	/**
	 * The trigger of the plan
	 * 
	 * @uml.property name="mTrigger"
	 * @uml.associationEnd multiplicity="(0 1)"
	 */
	private ICooDesire mTrigger;

	/**
	 * 
	 * The name of the service provided by the bpel process that implements the
	 * body of the plan
	 */
	private String mBodyID;

	/**
	 * The access specifier for the plan
	 */
	private ICooAccessSpecifier mAccessSpecifier;


	/**
	 * Constructor
	 * 
	 */
	public CooPlan() {
	}


	/**
	 * Constructor
	 * 
	 * @param plandID
	 * @param trigger
	 * @param bpelPlan
	 * @param accessSpecifier
	 */
	public CooPlan(String aPlanID, ICooDesire aTrigger, String aServiceName,
			ICooAccessSpecifier aAccessSpecifier) {
		mPlanID = aPlanID;
		mTrigger = aTrigger;
		mBodyID = aServiceName;
		mAccessSpecifier = aAccessSpecifier;
	}


	/**
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return mPlanID.hashCode();
	}


	public boolean equals(Object o) {
        if (o instanceof CooPlan)
            return mPlanID.equals(((CooPlan)o).getMPlanID());
        else
            return false;
	}


	public String toString() {
		String res = "(PLANID: ";
		res += this.mPlanID;
		res += (", TRIGGER=" + mTrigger);
		res += (", BODY=" + mBodyID);
		res += (", ACCSPEC=" + mAccessSpecifier);
		res += ")";

		return res;
	}


	/**
	 * 
	 * @uml.property name="mPlandID"
	 */
	public String getMPlanID() {
		return mPlanID;
	}


	/**
	 * 
	 * @uml.property name="mPlandID"
	 */
	public void setMPlanID(String planID) {
		mPlanID = planID;
	}


	/**
	 * 
	 * @uml.property name="mTrigger"
	 */
	public ICooDesire getMTrigger() {
		return mTrigger;
	}


	/**
	 * 
	 * @uml.property name="mTrigger"
	 */
	public void setMTrigger(ICooDesire trigger) {
		mTrigger = trigger;
	}


	public ICooAccessSpecifier getMAccessSpecifier() {
		return mAccessSpecifier;
	}


	public void setMAccessSpecifier(
            ICooAccessSpecifier accessSpecifier) {
		mAccessSpecifier = accessSpecifier;
	}


	public String getMServiceName() {
		return mBodyID;
	}


	public void setMServiceName(String serviceName) {
		mBodyID = serviceName;
	}
}
