package org.coows.aux;

import org.coows.CooAccessSpecifierPrivate;
import org.coows.CooAccessSpecifierPublic;
import org.coows.CooAgent;
import org.coows.CooBpelProcessBpr;
import org.coows.CooDesire;
import org.coows.CooPlan;

import org.coows.engine.CooEngine;
import org.coows.engine.CooEngineConfiguration;
import org.coows.engine.CooException;
import org.coows.engine.CooPersistenceManager;

import org.coows.util.CooAgentState;
import org.coows.util.CooBprFile;
import org.coows.util.CooSetOfPlans;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.IOException;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Vector;

/**
 * 
 */
public class PopulateDB {
	public void populateProcessesTable(CooEngineConfiguration aConf)
			throws CooException {

		CooBpelProcessBpr currProcess;

		Vector setOfProcesses = new Vector();

		try {
			currProcess = createProcess("runPlanService", 
                    System.getProperties().getProperty("user.home")
					+ "/temp/plan1Body.bpr");
            
			setOfProcesses.add(currProcess);

			CooPersistenceManager.serializeProcesses(aConf, setOfProcesses);

			System.out.println("[PopulateDB] processes added to db!");
		} catch (Exception e) {
			throw new CooException(
					"[PopulateDB] Error while populatin processes DB!\n"
							+ e.getMessage());
		}
	}


	/**
	 * @return
	 * 
	 * @throws IOException
	 */
	private CooBpelProcessBpr createProcess(
            String aServiceName,
			String aBprFilePath)
    	throws IOException {
        
		CooBpelProcessBpr res = new CooBpelProcessBpr();

		BufferedInputStream buf;
		buf = new BufferedInputStream(new FileInputStream(aBprFilePath));

		byte[] bytes = new byte[buf.available()];
		buf.read(bytes, 0, buf.available());

		ByteArrayInputStream bytearray = new ByteArrayInputStream(bytes);

		CooBprFile bpr = new CooBprFile(bytearray);

		res.setServiceName(aServiceName);
		res.setBprFile(bpr);

		return res;
	}


	private void populateAgentsTable(CooEngineConfiguration aConf)
			throws CooException {
        
		Collection aSetOfAgents = createSetOfAgentsState();
		CooPersistenceManager.serializeAgents(aConf, aSetOfAgents);
	}


	private Collection createSetOfAgentsState() throws CooException {
        
		Vector res = new Vector();
		CooAgent agent;
		CooAgentState agentState;
		CooSetOfPlans sop;
		CooPlan plan;

		for (int i = 0; i < 2; i++) {
			sop = new CooSetOfPlans();
			plan = new CooPlan("plan" + (3 * i), new CooDesire("trigger"
					+ (3 * i)), "runPlanService", CooAccessSpecifierPublic
					.getInstance());

			sop.addPlan(plan);

			plan = new CooPlan("plan" + ((3 * i) + 1), new CooDesire("trigger"
					+ ((3 * i) + 1)), "runPlanService",
					CooAccessSpecifierPrivate.getInstance());

			sop.addPlan(plan);

			plan = new CooPlan("plan" + ((3 * i) + 2), new CooDesire("trigger"
					+ ((3 * i) + 2)), "runPlanService",
					CooAccessSpecifierPublic.getInstance());

			sop.addPlan(plan);

			agentState = new CooAgentState("agent" + i, sop, new Vector(),
					CooAgent.RETRIEVAL_STRATEGY_NOLOCAL,
					CooAgent.ACQUISITION_STRATEGY_ADD);

			agent = new CooAgent(agentState);

			res.add(agent);
		}

		return res;
	}


	public static void main(String[] args) throws CooException {
        
		PopulateDB pd = new PopulateDB();

		CooEngine.initTEMP("cooEngineConf.xml", System.getProperties()
				.getProperty("user.home")
				+ "/temp");

		pd.populateProcessesTable(CooEngine.getMConfig());

		Hashtable setOfProcesses = CooPersistenceManager
				.restoreBpelProcesses(CooEngine.getMConfig());

		pd.populateAgentsTable(CooEngine.getMConfig());
		// TEST: Hashtable setOfAgents =
		// CooPersistenceManager.restoreAgents(aConf);
	}
}
