package org.coows.engine;

import java.sql.Connection;
import java.sql.DriverManager;

/**
 * Handler for the db connections
 */
public class CooDbConnectionsManager {
	/** Driver used for the connection */
	private String mJdbcDriver;

	/** The DB URL */
	private String mCooWSDbURL;

	/** The username allowed to access the mysql DB */
	private String mUser;

	/** The password */
	private String mPwd;


	public CooDbConnectionsManager(CooEngineConfiguration aConfig)
			throws CooException {
		mJdbcDriver = aConfig
				.getText("/config/DbConnectionManagerConf/jdbcdriver");
		mCooWSDbURL = aConfig.getText("/config/DbConnectionManagerConf/dburl");
		mUser = aConfig.getText("/config/DbConnectionManagerConf/dbusername");
		mPwd = aConfig.getText("/config/DbConnectionManagerConf/dbpassword");
	}


	public Connection getNewConnection() throws CooException {
		try {
			Class.forName(mJdbcDriver);

			return DriverManager.getConnection(mCooWSDbURL, mUser, mPwd);
		} catch (Exception e) {
			throw new CooException(
					"[CooDbConnectionsManager] Error connecting to db "
							+ mCooWSDbURL + "\n" + e.getMessage());
		}
	}
}
