package org.coows.engine;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jaxen.JaxenException;
import org.jaxen.XPath;

import org.jaxen.dom.DOMXPath;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

/**
 * 
 */
public class CooEngineConfiguration {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory
            .getLog(CooEngineConfiguration.class);
	
    private Document doc;

	private File configFile;


	/**
	 * @param configFile
	 * @param configDir
	 * 
	 * @throws CooException
	 */
	public CooEngineConfiguration(String configFileName, String configDir)
			throws CooException {
		try {
			DocumentBuilder builder = DocumentBuilderFactory.newInstance()
					.newDocumentBuilder();
			configFile = new File(configDir, configFileName);
			doc = builder.parse(configFile);
            
            if (logger.isDebugEnabled()) {
                logger.debug("CooEngineConfiguration() - " +
                        "CooWS engine configuration loaded!");
            }
            
		} catch (Exception e) {
			throw new CooException("[CooEngineConfiguration.CONSTRUCTOR]"
					+ " Error reading config file: " + e.getMessage());
		}
	}


	/**
	 * Given a lookup <var>xpath</var> and an attribute name, returns the value
	 * of that attribute.
	 * 
	 * @param xpath
	 *            an XPath expression
	 * @param attrName
	 *            an attribute name
	 * 
	 * @return the value of the text inside the specified element
	 */
	public String getAttr(String xpath, String attrName) throws CooException {
		try {
			return get(xpath + "/@" + attrName);
		} catch (Exception e) {
			throw new CooException(
					"[CooEngine ERROR] unable to get the value of the value "
							+ xpath + "/@" + attrName + " from config file");
		}
	}


	/**
	 * Given a lookup <var>xpath</var>, returns the value of the text inside
	 * the specified element.
	 * 
	 * @param xpath
	 *            an XPath expression
	 * 
	 * @return the value of the text inside the specified element
	 */
	public String getText(String xpath) throws CooException {
		try {
			return get(xpath + "/text()[1]");
		} catch (Exception e) {
			throw new CooException("[CooEngine ERROR] unable to get the value "
					+ xpath + " from config file");
		}
	}


	/**
	 * Given an <var>xpath</var>, returns the value of that element.
	 * 
	 * @param xpath
	 *            an XPath expression
	 * 
	 * @return the value
	 */
	public String get(String xpath) throws CooException {
		XPath xp;

		try {
			xp = new DOMXPath(xpath);

			Node node = (Node) xp.selectSingleNode(doc);

			return node.getNodeValue().trim();
		} catch (JaxenException e) {
			throw new CooException("[CooEngine ERROR] unable to get the value "
					+ xpath + " from config file");
		}
	}
}
