package org.coows.engine;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.thoughtworks.xstream.*;

import org.coows.CooAgent;
import org.coows.CooBpelProcessBpr;
import org.coows.ICooBpelProcess;
import org.coows.util.CooAgentState;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;

/**
 * Stores/Retrieves the state from the DB
 *
 */
public class CooPersistenceManager {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory
            .getLog(CooPersistenceManager.class);

    /**
     * Instantiates the agents from a previously saved state 
     * @param aConf
     * @return a table of agents
     * @throws CooException
     */
	public static Hashtable<String,CooAgent> 
        restoreAgents(CooEngineConfiguration aConf)
			throws CooException {
        if (logger.isDebugEnabled()) {
            logger.debug("restoreAgents(): reloading agents state...");
        }
        
		Hashtable res = new Hashtable<String,CooAgent>();
		XStream xstream = new XStream();

		Statement st;
		ResultSet rs;
		Connection dbConn;
		CooAgentState currAgentState;
		CooAgent currAgent;

		// debug("Getting connection to the db....");
		dbConn = (new CooDbConnectionsManager(aConf)).getNewConnection();

		try {
			st = dbConn.createStatement();

			// debug("Querying for agents states...");
			rs = st.executeQuery("SELECT * FROM agentsXML");
            
            if (logger.isDebugEnabled()) {
                logger.debug("restoreAgents() - " +
                        "agents states retrieved! Now initialiazing!");
            }

			while (rs.next()) {
                //logger.debug("yahooooooooooooooooooooooooooooooooooo");
				currAgentState = (CooAgentState) 
                    xstream.fromXML(rs.getString("serializedagent"));
                //logger.debug("yahiiiiiiiiiiiiiiiiii");
				currAgent = new CooAgent(currAgentState);
                //logger.debug("yahiaaaaaaaaaaaaaaaai");
				res.put(currAgent.getMAgentID(), currAgent);
			}

			dbConn.close();

            if (logger.isDebugEnabled()) {
                logger.debug("restoreAgents() - Agents state initialized!");
            }
            
		} catch (SQLException e) {
			throw new CooException(
					"[CooPersistenceManager] Error while deserializing agents"
							+ " from the db!\n" + e.getMessage());
		}
        
        if (logger.isDebugEnabled())
        {
            logger.debug("restoreAgents() - Agents states reloaded!");
        }

		return res;
	}


    /**
     * Retrieve the BPEL processes saved to the db 
     * @param aConf
     * @return
     * @throws CooException
     */
	public static Hashtable<String,ICooBpelProcess> 
        restoreBpelProcesses(CooEngineConfiguration aConf)
			throws CooException {
        
		Hashtable<String,ICooBpelProcess> res = 
            new Hashtable<String,ICooBpelProcess>();
		XStream xstream = new XStream();

		Statement st;
		ResultSet rs;
		Connection dbConn;
		ICooBpelProcess currProcess;

		dbConn = (new CooDbConnectionsManager(aConf)).getNewConnection();

		try {
			st = dbConn.createStatement();
			rs = st.executeQuery("SELECT * FROM bpelProcessesXML");

			while (rs.next()) {
				currProcess = (CooBpelProcessBpr) xstream.fromXML(
                        rs.getString("serializedBpelProcess"));
				res.put(currProcess.getServiceName(), currProcess);
				System.out.println("[CooPersistenceManager] Process "
								+ currProcess.getServiceName() + " retrieved.");
			}

			dbConn.close();
		} catch (SQLException e) {
			throw new CooException("[CooPersistenceManager]" +
					"Error while deserializing agents "	+ 
					"from the db!\n" + e.getMessage());
		}

		return res;
	}


    /**
     * 
     * @param aConf
     * @param agents
     * @throws CooException
     */
	public static void serializeAgents(
            CooEngineConfiguration aConf,
			Collection agents)
    	throws CooException {
        
		Statement st;
		PreparedStatement pSt;
		ResultSet rs;
		CooAgentState currAgentState;
		String agent2xml;

		XStream xstream = new XStream();

		Connection dbConn = (new CooDbConnectionsManager(aConf))
				.getNewConnection();

		try {
			st = dbConn.createStatement();
			st.execute("DELETE FROM agentsXML WHERE 1");
			st.close();

			pSt = dbConn.prepareStatement("INSERT INTO agentsXML VALUES (?)");

            //for (CooAgentState as : agents) {}
			for (Iterator i = agents.iterator(); i.hasNext();) {
				currAgentState = ((CooAgent) i.next()).getAgentState();
				agent2xml = xstream.toXML(currAgentState);
				// debug("AGENT " + currAgent.getMAgentID()
				// + " SERIALIZED:\n" + agent2xml);
				// pSt.setString(1, currAgent.getMAgentID());
				pSt.setString(1, agent2xml);
				pSt.executeUpdate();
			}

            if (logger.isDebugEnabled())
            {
                logger.debug("serializeAgents() done!");
            }

			pSt.close();
			dbConn.close();
		} catch (SQLException e) {
			throw new CooException(
					"[CooPersistenceManager] Error while serializing agents"
							+ "to the db!\n" + e.getMessage());
		}
	}


    /**
     * 
     * @param aConf
     * @param processes
     * @throws CooException
     */
	public static void serializeProcesses(CooEngineConfiguration aConf,
			Collection processes) throws CooException {
		Statement st;
		PreparedStatement pSt;
		ResultSet rs;
		CooBpelProcessBpr currProcess;
		String process2xml;

		XStream xstream = new XStream();

		Connection dbConn = (new CooDbConnectionsManager(aConf))
				.getNewConnection();

		try {
			st = dbConn.createStatement();
			st.execute("DELETE FROM bpelProcessesXML WHERE 1");
			st.close();

			pSt = dbConn.prepareStatement(
                    "INSERT INTO bpelProcessesXML VALUES (?,?)");

			for (Iterator i = processes.iterator(); i.hasNext();) {
				currProcess = (CooBpelProcessBpr) i.next();
				process2xml = xstream.toXML(currProcess);
				pSt.setString(1, currProcess.getServiceName());
				pSt.setString(2, process2xml);
				pSt.executeUpdate();
			}

			dbConn.close();
            
            if (logger.isDebugEnabled())
            {
                logger.debug("serializeProcesses() done!");
            }
            
		} catch (SQLException e) {
			throw new CooException(
					"[CooPersistentManager] Error while serializing processes"
							+ "to the db!\n" + e.getMessage());
		}
	}
}
