package org.coows.test;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;
import javax.xml.rpc.ServiceException;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.ser.BeanDeserializerFactory;
import org.apache.axis.encoding.ser.BeanSerializerFactory;
import org.coows.CooAccessSpecifierOnlyTrusted;
import org.coows.CooAccessSpecifierPrivate;
import org.coows.CooAccessSpecifierPublic;
import org.coows.CooBpelProcessBpr;
import org.coows.CooDesire;
import org.coows.CooEventProvided;
import org.coows.CooEventRemote;
import org.coows.CooEventRequested;
import org.coows.CooPlan;
import org.coows.ICooEvent;
import org.coows.ser.CooBprFileDeserializerFactory;
import org.coows.ser.CooBrpFileSerializerFactory;
import org.coows.util.CooBprFile;
import org.coows.util.CooRequestID;
import org.coows.util.CooSetOfPlans;

public class DispatchEventProvidedTestClient {
    public static void main(String[] args)
    {
        Vector calls = new Vector();
        Vector requested = new Vector();
        
        try
        {
            Call call = createCall();
            CooEventProvided prov = new CooEventProvided("agent0","agent1",null,null);
            CooPlan aPlan = new CooPlan("aPlanID",new CooDesire("aTrigger"), "runPlan", CooAccessSpecifierPublic.getInstance());
            CooSetOfPlans aSop = new CooSetOfPlans();
            aSop.addPlan(aPlan);
            prov.setMProvidedPlans(aSop);
            
            for (int i=3; i<4; i++)
            {
                prov.setMReqID(new CooRequestID("intentionID","instanceID"));
                System.out.println("Sending event "+prov);
                call.invoke(new Object[]{new CooEventRemote(prov,"localhost")});
            }
        }
        catch(Exception ex)
        {
            System.out.println("Error invoking WS: " + ex.getMessage());
            ex.printStackTrace();
        }
    } // end of main
    
    public static Call createCall() throws ServiceException, MalformedURLException
    {
        Call call = (Call)new Service().createCall();
        call.setTargetEndpointAddress(new URL("http://localhost:4444/active-bpel/services/CooWS"));

        // Registering type mapping
        call.registerTypeMapping(CooDesire.class, new QName("urn:coows", "Desire"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooPlan.class, new QName("urn:coows", "Plan"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooAccessSpecifierPublic.class, new QName("urn:coows", "AccessSpecifierPublic"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooAccessSpecifierPrivate.class, new QName("urn:coows", "AccessSpecifierPrivate"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooAccessSpecifierOnlyTrusted.class, new QName("urn:coows", "AccessSpecifierOnlyTrusted"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooBpelProcessBpr.class, new QName("urn:coows", "BpelProcessBpr"), BeanSerializerFactory.class, BeanDeserializerFactory.class);            
        call.registerTypeMapping(CooBprFile.class, new QName("urn:coows", "BprFile"), CooBrpFileSerializerFactory.class, CooBprFileDeserializerFactory.class);
        call.registerTypeMapping(ICooEvent.class, new QName("urn:coows", "Event"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooEventRequested.class, new QName("urn:coows", "EventRequested"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooEventProvided.class, new QName("urn:coows", "EventProvided"), BeanSerializerFactory.class, BeanDeserializerFactory.class);        
        call.registerTypeMapping(CooEventRemote.class, new QName("urn:coows", "EventRemote"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooRequestID.class, new QName("urn:coows", "RequestID"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        call.registerTypeMapping(CooSetOfPlans.class, new QName("urn:coows", "SetOfPlans"), BeanSerializerFactory.class, BeanDeserializerFactory.class);
        
        QName remoteEventQName = new QName("urn:coows", "EventRemote");
        call.addParameter(remoteEventQName, remoteEventQName, CooEventRequested.class, ParameterMode.IN);
        call.setOperationStyle("rpc");

        call.setOperationName(new QName("CooWS", "dispatchEventRemote"));
        
        return call;

    }
}
