package org.coows.util;

import java.util.Vector;

/**
 * The agent state.
 * Holds the info to be dumped to the db
 */
public class CooAgentState
{
    /** The agent ID */
    private String mAgentID;

    /**
     * The agent plan library
     * 
     * @uml.property name="mSop"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private CooSetOfPlans mSop;

    
    /** The list of known agents */
    private Vector mPartners;
    
    /** The retrieval strategy */
    private int mRetrievalStrategy;
    
    /** The acquisition strategy */
    private int mAcquisitionStrategy;

    /**
     * Constructor 
     * @param agentID
     * @param sop
     * @param partners
     * @param retrievalStrategy
     * @param acquisitionStrategy
     */
    public CooAgentState(
            String agentID, 
            CooSetOfPlans sop, 
            Vector partners,
            int retrievalStrategy,
            int acquisitionStrategy)
    {
        mAgentID = agentID;
        mSop = sop;
        mPartners = partners;
        mRetrievalStrategy = retrievalStrategy;
        mAcquisitionStrategy = acquisitionStrategy;
    }

    /**
     * 
     * @uml.property name="mAgentID"
     */
    public String getMAgentID() {
        return mAgentID;
    }

    /**
     * 
     * @uml.property name="mAgentID"
     */
    public void setMAgentID(String agentID) {
        mAgentID = agentID;
    }

    /**
     * 
     * @uml.property name="mPartners"
     */
    public Vector getMPartners() {
        return mPartners;
    }

    /**
     * 
     * @uml.property name="mPartners"
     */
    public void setMPartners(Vector partners) {
        mPartners = partners;
    }

    /**
     * 
     * @uml.property name="mSop"
     */
    public CooSetOfPlans getMSop() {
        return mSop;
    }

    /**
     * 
     * @uml.property name="mSop"
     */
    public void setMSop(CooSetOfPlans sop) {
        mSop = sop;
    }

    /**
     * 
     * @uml.property name="mAcquisitionStrategy"
     */
    public int getMAcquisitionStrategy() {
        return mAcquisitionStrategy;
    }

    /**
     * 
     * @uml.property name="mAcquisitionStrategy"
     */
    public void setMAcquisitionStrategy(int as) {
        mAcquisitionStrategy = as;
    }

    /**
     * 
     * @uml.property name="mRetrievalStrategy"
     */
    public int getMRetrievalStrategy() {
        return mRetrievalStrategy;
    }

    /**
     * 
     * @uml.property name="mRetrievalStrategy"
     */
    public void setMRetrievalStrategy(int retrievalStrategy) {
        mRetrievalStrategy = retrievalStrategy;
    }

}
