package org.coows.util;

import org.coows.CooPlan;
import org.coows.ICooDesire;

import java.util.Collection;
import java.util.HashSet;
import java.util.Vector;

/**
 * Wrap a set of plans
 */
public class CooSetOfPlans {
    /** Contains a */
    private HashSet<CooPlan> mPlans;

    public CooSetOfPlans() {
        mPlans = new HashSet<CooPlan>();
    }

    public boolean isEmpty() {
        return mPlans.isEmpty();
    }

    /**
     * Add a plan to the set
     * 
     * @param aPlan
     */
    public void addPlan(CooPlan aPlan) {
        mPlans.add(aPlan);
    }

    /**
     * Add a set of plans to the current set
     * 
     * @param aSop
     */
    public void addPlans(CooSetOfPlans aSop) {
        mPlans.addAll(aSop.getCollection());
    }

    /**
     * Remove a plan from the set
     * 
     * @param aPlan
     *            The plan to be removed
     */
    public void removePlan(CooPlan aPlan) {
        mPlans.remove(aPlan);
    }

    public CooPlan getAPlan() {
        return mPlans.iterator().next();
    }

    public Collection<CooPlan> getCollection() {
        return mPlans;
    }

    /**
     * Selects and returns a set of relevant plans for the given agent and
     * desire
     * 
     * @param anAgentID
     * @param aDesire
     * @return
     */
    public CooSetOfPlans getRelevantForAgent(String anAgentID,
            ICooDesire aDesire) {
        CooSetOfPlans res = new CooSetOfPlans();
        

        // Check all the plans in the set
        for (CooPlan curr : mPlans) {
            // if the agent is trusted
            if (curr.getMAccessSpecifier().isTrusted(anAgentID))
            // and the trigger of the plan matches with the desire
                if (curr.getMTrigger().match(aDesire))
                // add the plan to the result set
                    res.addPlan(curr);
        }

        return res;
    }

    /**
     * Selects and returns a set of relevant plans for the agent owner
     * 
     * @param aDesire
     * @return
     */
    public CooSetOfPlans getRelevant(ICooDesire aDesire) {
        CooSetOfPlans res = new CooSetOfPlans();
        
        // Check all the plans in the set
        for (CooPlan curr : mPlans) {
            if (curr.getMTrigger().match(aDesire))
            // add the plan to the result set
                res.addPlan(curr);
        }

        return res;
    }

    /**
     * Get the size of the set
     */
    public int size() {
        return mPlans.size();
    }

    public String toString() {

        String res = "SetOfPlans=[";

        for (CooPlan curr : mPlans)
            res += " " + curr;

        return res + "]";
    }

    public HashSet<CooPlan> getMPlans() {
        return mPlans;
    }

    public void setMPlans(HashSet<CooPlan> plans) {
        mPlans = plans;
    }

    public void removePlans(CooSetOfPlans aSop) {
        mPlans.removeAll(aSop.getCollection());
    }

    /**
     * Update the set with a collection of provided plans.<br>
     * (removing the plans that match with one of the provided)
     * 
     * @param aSop
     */
    public void replace(CooSetOfPlans aSop) {
        Vector<CooPlan> toDel = new Vector<CooPlan>();

        // Detect all the matching plans already in the set
        for (CooPlan curr : mPlans) {
            if (!getRelevant(curr.getMTrigger()).isEmpty())
                toDel.add(curr);
        }

        // Remove the matching plans
        mPlans.removeAll(toDel);

        // Add the new plans
        addPlans(aSop);
    }

}
