package org.coows.work;

import java.util.LinkedList;

import org.apache.commons.pool.impl.GenericObjectPool;
import org.coows.CooAgent;
import org.coows.CooGlobalEventsQueue;
import org.coows.ICooEvent;
import org.coows.engine.CooEngine;


public abstract class CooAbstractWorker
    extends Thread
{
    protected boolean running = false;
    protected boolean stopped = false;
    protected boolean done = true;
    protected GenericObjectPool pool = null;

    public CooAbstractWorker()
    {
        this.setDaemon(true);
    }

    /**
     * 
     * @uml.property name="pool"
     */
    public void setPool(GenericObjectPool pool) {
        this.pool = pool;
    }

    /**
     * 
     * @uml.property name="stopped"
     */
    public void setStopped(boolean b) {
        stopped = b;
    }

    public void reset()
    {
        this.done = true;
    }

    public void run()
    {
        running = true;

        while (!stopped)
        {
            if (done)
            {
                synchronized (this)
                {
                    try
                    {
                        this.wait();
                    }
                    catch (InterruptedException e)
                    {
                        stopped = true;
                    }
                }
            }
            else
            {
                // there is a task....let us execute it.
                try
                {
                    execute();
                }
                catch (Exception e)
                {
                }
                finally
                {
                    ICooWorkItem w = getNextWorkItem();

                    if (null != w)
                    {
                        processNextWorkItem(w);
                    }
                    else
                    {
                        reset();
                        returnToPool();
                    }
                }
            }
        }
    }

    protected void handle()
    {
        this.done = false;

        if (!running)
        {
            // If this is the first time, then kick off the thread.
            this.setDaemon(true);
            this.start();
        }
        else
        {
            // we already have a thread running so wakeup the waiting thread.
            this.notifyAll();
        }
    }

    private void returnToPool()
    {
        if (pool != null)
        {
            try
            {
                pool.returnObject(this);
            }
            catch (Exception e1)
            {
            }

            this.pool = null;
        }
    }
    
    private ICooWorkItem getNextWorkItem()
    {
        ICooEvent e = null;
        LinkedList qe = getQueuedEvents();

        synchronized (qe)
        {
            if (!qe.isEmpty())
            {
                e = (ICooEvent) qe.remove(0);
            }
        }
        
        return e;
    }

    
    
    protected abstract void processNextWorkItem(ICooWorkItem w);
    protected abstract void execute();
    protected abstract LinkedList getQueuedEvents();
}
